/*
 Basic ESP8266 MQTT example

 This sketch demonstrates the capabilities of the pubsub library in combination
 with the ESP8266 board/library.

 It connects to an MQTT server then:
  - publishes "hello world" to the topic "outTopic" every two seconds
  - subscribes to the topic "inTopic", printing out any messages
    it receives. NB - it assumes the received payloads are strings not binary
  - If the first character of the topic "inTopic" is an 1, switch ON the ESP Led,
    else switch it off

 It will reconnect to the server if the connection is lost using a blocking
 reconnect function. See the 'mqtt_reconnect_nonblocking' example for how to
 achieve the same result without blocking the main loop.

 To install the ESP8266 board, (using Arduino 1.6.4+):
  - Add the following 3rd party board manager under "File -> Preferences -> Additional Boards Manager URLs":
       http://arduino.esp8266.com/stable/package_esp8266com_index.json
  - Open the "Tools -> Board -> Board Manager" and click install for the ESP8266"
  - Select your ESP8266 in "Tools -> Board"

*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <OneWire.h>                // https://github.com/PaulStoffregen/OneWire
#include <DallasTemperature.h>      // https://github.com/milesburton/Arduino-Temperature-Control-Library
#include <DHT.h>                    // über "Biblitheken verwalten" eingebunden

const char* ssid        = "Your ssid";
const char* password    = "Your password";
const char* mqtt_server = "IP of your MQTT server";
const int   RATE        = 5000;    // 5 sec. time between measurement
#define ONE_WIRE_BUS 2             // OneWire bus on GPIO 2
#define DHTPIN       12            // DHT-Pin at GPIO 12
#define DHTTYPE      DHT22         // used Chip
DHT dht(DHTPIN, DHTTYPE);

long lastMsg;                      // Remember time of last measure

WiFiClient espClient;              // definitions
PubSubClient client(espClient);

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

void setup() {                     // setup routine
  Serial.begin(115200);
  setup_wifi();   
  client.setServer(mqtt_server, 1883);  // Init connection to Broker
  client.setCallback(callback);
  dht.begin();
}

void setup_wifi() {               // Connect to WLAN
  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");             // is connected
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}
                                  // in case subscribe arrives
void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();
}
                                 // Check connection to broker
void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("ESP8266Client")) {
      Serial.println("connected");
      client.publish("outTopic", "\r\nStart Messung\r\n");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
       delay(5000);
    }
  }
}

void loop() {
  char  tmp[20];                 // String temperature DS18B20    
  char  ana[20];                 // String light
  char  dtmpst [20];             // String temp DHT
  char  dhumst [20];             // String humi DHT
  char  anaOut[100];             // String for both
  float tempVal;                 // temperature DS18B20
  int   anaInt;                  // Light
  float anaFlo;                  // Light
  float dhttmp;                  // temperature DHT
  float dhthum;                  // humidity DHT

  if (!client.connected()) {
    reconnect();
  }
  client.loop();

  long now = millis();
  if (now - lastMsg > RATE) {
    lastMsg = now;                 // Do next measurement
    sensors.requestTemperatures(); 
    tempVal = sensors.getTempCByIndex(0); // get temperature
    dtostrf(tempVal, 2, 2, tmp);   // convert to string

    dhttmp = dht.readTemperature();  // read DHT values
    dhthum = dht.readHumidity();
    dtostrf(dhttmp, 2, 2, dtmpst); // convert to string
    dtostrf(dhthum, 2, 2, dhumst); // convert to string
    
    anaInt = analogRead(A0);       // get analog value
    anaFlo = 100.0 - (anaInt/2.5); // calculate to percent base
    dtostrf(anaFlo, 2, 2, ana);    // convert to string
    
    sprintf(anaOut, "TempDS = %s  TempDHT = %s  HumDHT = %s   Licht = %s %\n\r", tmp, dtmpst, dhumst, ana);
    Serial.println(anaOut);        // print all on serial monitor
    client.publish("outTopic", anaOut);  // publish all to broker
  }
}
